@@ -118,42 +118,28 @@ impl AttributeBufferDataV8 {
     }
 }
 
-// TODO: Simplify with const generics?
-fn get_f16_vector4(vector: &[f32; 4]) -> [f16; 4] {
-    let [x, y, z, w] = vector;
-    [
-        f16::from_f32(*x),
-        f16::from_f32(*y),
-        f16::from_f32(*z),
-        f16::from_f32(*w),
-    ]
+fn get_f16_vector<const N: usize>(vector: &[f32; N]) -> [f16; N] {
+    let mut output = [f16::ZERO; N];
+    for i in 0..N {
+        output[i] = f16::from_f32(vector[i]);
+    }
+    output
 }
 
-fn get_f16_vector2(vector: &[f32; 2]) -> [f16; 2] {
-    let [x, y] = vector;
-    [f16::from_f32(*x), f16::from_f32(*y)]
+fn get_clamped_u8_vector<const N: usize>(vector: &[f32; N]) -> [u8; N] {
+    let mut output = [0u8; N];
+    for i in 0..N {
+        output[i] = get_u8_clamped(vector[i]);
+    }
+    output
 }
 
-fn get_clamped_u8_vector4(vector: &[f32; 4]) -> [u8; 4] {
-    let [x, y, z, w] = vector;
-    [
-        get_u8_clamped(*x),
-        get_u8_clamped(*y),
-        get_u8_clamped(*z),
-        get_u8_clamped(*w),
-    ]
+fn get_f16_vectors<const N: usize>(vector: &[[f32; N]]) -> Vec<[f16; N]> {
+    vector.iter().map(get_f16_vector).collect()
 }
 
-fn get_f16_vector2s(vector: &[[f32; 2]]) -> Vec<[f16; 2]> {
-    vector.iter().map(get_f16_vector2).collect()
-}
-
-fn get_f16_vector4s(vector: &[[f32; 4]]) -> Vec<[f16; 4]> {
-    vector.iter().map(get_f16_vector4).collect()
-}
-
-fn get_clamped_u8_vector4s(vector: &[[f32; 4]]) -> Vec<[u8; 4]> {
-    vector.iter().map(get_clamped_u8_vector4).collect()
+fn get_clamped_u8_vectors<const N: usize>(vector: &[[f32; N]]) -> Vec<[u8; N]> {
+    vector.iter().map(get_clamped_u8_vector).collect()
 }
 
 fn get_position_data_v8(data: &[AttributeData]) -> Vec<AttributeBufferDataV8> {
@@ -191,7 +177,7 @@ fn get_vector_data_v8(data: &[AttributeData]) -> Vec<AttributeBufferDataV8> {
         .map(|a| match &a.data {
             VectorData::Vector2(v) => AttributeBufferDataV8::Float2(v.clone()),
             VectorData::Vector3(v) => AttributeBufferDataV8::Float3(v.clone()),
-            VectorData::Vector4(v) => AttributeBufferDataV8::HalfFloat4(get_f16_vector4s(v)),
+            VectorData::Vector4(v) => AttributeBufferDataV8::HalfFloat4(get_f16_vectors(v)),
         })
         .collect()
 }
@@ -203,7 +189,7 @@ fn get_vector_data_v9(data: &[AttributeData]) -> Vec<(String, AttributeBufferDat
             VectorData::Vector3(v) => (a.name.clone(), AttributeBufferDataV8::Float3(v.clone())),
             VectorData::Vector4(v) => (
                 a.name.clone(),
-                AttributeBufferDataV8::HalfFloat4(get_f16_vector4s(v)),
+                AttributeBufferDataV8::HalfFloat4(get_f16_vectors(v)),
             ),
         })
         .collect()
@@ -214,12 +200,12 @@ fn get_vector_data_v10(data: &[AttributeData]) -> Vec<(String, AttributeBufferDa
         .map(|a| match &a.data {
             VectorData::Vector2(v) => (
                 a.name.clone(),
-                AttributeBufferDataV10::HalfFloat2(get_f16_vector2s(v)),
+                AttributeBufferDataV10::HalfFloat2(get_f16_vectors(v)),
             ),
             VectorData::Vector3(v) => (a.name.clone(), AttributeBufferDataV10::Float3(v.clone())),
             VectorData::Vector4(v) => (
                 a.name.clone(),
-                AttributeBufferDataV10::HalfFloat4(get_f16_vector4s(v)),
+                AttributeBufferDataV10::HalfFloat4(get_f16_vectors(v)),
             ),
         })
         .collect()
@@ -230,7 +216,7 @@ fn get_color_data_v8(data: &[AttributeData]) -> Vec<AttributeBufferDataV8> {
         .map(|a| match &a.data {
             VectorData::Vector2(v) => AttributeBufferDataV8::Float2(v.clone()),
             VectorData::Vector3(v) => AttributeBufferDataV8::Float3(v.clone()),
-            VectorData::Vector4(v) => AttributeBufferDataV8::Byte4(get_clamped_u8_vector4s(v)),
+            VectorData::Vector4(v) => AttributeBufferDataV8::Byte4(get_clamped_u8_vectors(v)),
         })
         .collect()
 }
@@ -242,7 +228,7 @@ fn get_color_data_v9(data: &[AttributeData]) -> Vec<(String, AttributeBufferData
             VectorData::Vector3(v) => (a.name.clone(), AttributeBufferDataV8::Float3(v.clone())),
             VectorData::Vector4(v) => (
                 a.name.clone(),
-                AttributeBufferDataV8::Byte4(get_clamped_u8_vector4s(v)),
+                AttributeBufferDataV8::Byte4(get_clamped_u8_vectors(v)),
             ),
         })
         .collect()
@@ -253,12 +239,12 @@ fn get_color_data_v10(data: &[AttributeData]) -> Vec<(String, AttributeBufferDat
         .map(|a| match &a.data {
             VectorData::Vector2(v) => (
                 a.name.clone(),
-                AttributeBufferDataV10::HalfFloat2(get_f16_vector2s(v)),
+                AttributeBufferDataV10::HalfFloat2(get_f16_vectors(v)),
             ),
             VectorData::Vector3(v) => (a.name.clone(), AttributeBufferDataV10::Float3(v.clone())),
             VectorData::Vector4(v) => (
                 a.name.clone(),
-                AttributeBufferDataV10::Byte4(get_clamped_u8_vector4s(v)),
+                AttributeBufferDataV10::Byte4(get_clamped_u8_vectors(v)),
             ),
         })
         .collect()
@@ -484,7 +470,7 @@ fn add_attributes_v9(
             (AttributeUsageV9::Tangent, 0) => "map1",
             (AttributeUsageV9::Binormal, 0) => "map1",
             (AttributeUsageV9::Binormal, 1) => "uvSet",
-            _ => &attribute_name,
+            _ => attribute_name,
         };
 
         let attribute = MeshAttributeV9 {
@@ -518,7 +504,7 @@ fn add_attributes_v10(
             (AttributeUsageV9::Tangent, 0) => "map1",
             (AttributeUsageV9::Binormal, 0) => "map1",
             (AttributeUsageV9::Binormal, 1) => "uvSet",
-            _ => &attribute_name,
+            _ => attribute_name,
         };
 
         let attribute = MeshAttributeV10 {
@@ -669,13 +655,10 @@ pub(crate) fn write_attributes<W: Write + Seek>(
 
 #[cfg(test)]
 mod tests {
+    use super::*;
+    use hexlit::hex;
     use std::io::Cursor;
 
-    use crate::hex_bytes;
-
-    use super::*;
-
-    // TODO: Modify the functions to just take &[VectorData] instead of &[AttributeData]?
     fn create_attribute_data(data: &[VectorData]) -> Vec<AttributeData> {
         data.iter()
             .map(|data| AttributeData {
@@ -693,7 +676,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float2(vec![[0.0, 1.0]])
             )],
-            get_position_data_v9(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_position_data_v9(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
@@ -703,7 +686,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float3(vec![[0.0, 1.0, 2.0]])
             )],
-            get_position_data_v9(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_position_data_v9(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -713,7 +696,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float4(vec![[0.0, 1.0, 2.0, 3.0]])
             )],
-            get_position_data_v9(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_position_data_v9(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 1.0, 2.0, 3.0
             ]])]))
         );
@@ -727,7 +710,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::Float2(vec![[0.0, 1.0]])
             )],
-            get_position_data_v10(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_position_data_v10(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
@@ -737,7 +720,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::Float3(vec![[0.0, 1.0, 2.0]])
             )],
-            get_position_data_v10(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_position_data_v10(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -747,7 +730,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::Float4(vec![[0.0, 1.0, 2.0, 3.0]])
             )],
-            get_position_data_v10(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_position_data_v10(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 1.0, 2.0, 3.0
             ]])]))
         );
@@ -761,7 +744,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float2(vec![[0.0, 1.0]])
             )],
-            get_vector_data_v9(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_vector_data_v9(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
@@ -771,7 +754,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float3(vec![[0.0, 1.0, 2.0]])
             )],
-            get_vector_data_v9(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_vector_data_v9(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -786,7 +769,7 @@ mod tests {
                     f16::from_f32(3.0)
                 ]])
             )],
-            get_vector_data_v9(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_vector_data_v9(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 1.0, 2.0, 3.0
             ]])]))
         );
@@ -800,7 +783,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::HalfFloat2(vec![[f16::from_f32(0.0), f16::from_f32(1.0),]])
             )],
-            get_vector_data_v10(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_vector_data_v10(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
@@ -810,7 +793,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::Float3(vec![[0.0, 1.0, 2.0]])
             )],
-            get_vector_data_v10(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_vector_data_v10(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -825,7 +808,7 @@ mod tests {
                     f16::from_f32(3.0)
                 ]])
             )],
-            get_vector_data_v10(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_vector_data_v10(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 1.0, 2.0, 3.0
             ]])]))
         );
@@ -839,7 +822,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float2(vec![[0.0, 1.0]])
             )],
-            get_color_data_v9(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_color_data_v9(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
@@ -849,7 +832,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Float3(vec![[0.0, 1.0, 2.0]])
             )],
-            get_color_data_v9(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_color_data_v9(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -859,7 +842,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV8::Byte4(vec![[0u8, 128u8, 255u8, 255u8]])
             )],
-            get_color_data_v9(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_color_data_v9(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 0.5, 1.0, 2.0
             ]])]))
         );
@@ -873,7 +856,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::HalfFloat2(vec![[f16::from_f32(0.0), f16::from_f32(1.0)]])
             )],
-            get_color_data_v10(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_color_data_v10(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
@@ -883,7 +866,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::Float3(vec![[0.0, 1.0, 2.0]])
             )],
-            get_color_data_v10(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_color_data_v10(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -893,7 +876,7 @@ mod tests {
                 String::new(),
                 AttributeBufferDataV10::Byte4(vec![[0u8, 128u8, 255u8, 255u8]])
             )],
-            get_color_data_v10(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_color_data_v10(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 0.5, 1.0, 2.0
             ]])]))
         );
@@ -904,21 +887,21 @@ mod tests {
         // Check that positions use the largest available floating point type.
         assert_eq!(
             vec![AttributeBufferDataV8::Float2(vec![[0.0, 1.0]])],
-            get_position_data_v8(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_position_data_v8(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
 
         assert_eq!(
             vec![AttributeBufferDataV8::Float3(vec![[0.0, 1.0, 2.0]])],
-            get_position_data_v8(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_position_data_v8(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
 
         assert_eq!(
             vec![AttributeBufferDataV8::Float4(vec![[0.0, 1.0, 2.0, 3.0]])],
-            get_position_data_v8(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_position_data_v8(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 1.0, 2.0, 3.0
             ]])]))
         );
@@ -929,14 +912,14 @@ mod tests {
         // Check that vectors use the smallest available floating point type.
         assert_eq!(
             vec![AttributeBufferDataV8::Float2(vec![[0.0, 1.0]])],
-            get_vector_data_v8(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_vector_data_v8(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
 
         assert_eq!(
             vec![AttributeBufferDataV8::Float3(vec![[0.0, 1.0, 2.0]])],
-            get_vector_data_v8(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_vector_data_v8(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -948,7 +931,7 @@ mod tests {
                 f16::from_f32(2.0),
                 f16::from_f32(3.0)
             ]])],
-            get_vector_data_v8(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_vector_data_v8(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 1.0, 2.0, 3.0
             ]])]))
         );
@@ -959,14 +942,14 @@ mod tests {
         // Check that color sets use the smallest available type.
         assert_eq!(
             vec![AttributeBufferDataV8::Float2(vec![[0.0, 1.0]])],
-            get_color_data_v8(&create_attribute_data(&vec![VectorData::Vector2(vec![[
+            get_color_data_v8(&create_attribute_data(&[VectorData::Vector2(vec![[
                 0.0, 1.0
             ]])]))
         );
 
         assert_eq!(
             vec![AttributeBufferDataV8::Float3(vec![[0.0, 1.0, 2.0]])],
-            get_color_data_v8(&create_attribute_data(&vec![VectorData::Vector3(vec![[
+            get_color_data_v8(&create_attribute_data(&[VectorData::Vector3(vec![[
                 0.0, 1.0, 2.0
             ]])]))
         );
@@ -975,7 +958,7 @@ mod tests {
             vec![AttributeBufferDataV8::Byte4(vec![[
                 0u8, 128u8, 255u8, 255u8
             ]])],
-            get_color_data_v8(&create_attribute_data(&vec![VectorData::Vector4(vec![[
+            get_color_data_v8(&create_attribute_data(&[VectorData::Vector4(vec![[
                 0.0, 0.5, 1.0, 2.0
             ]])]))
         );
@@ -1508,7 +1491,7 @@ mod tests {
         )];
         write_attributes(&buffer_info, &mut [&mut buffer0], &[0]).unwrap();
 
-        assert_eq!(&hex_bytes("0000803F 00000040 00004040"), buffer0.get_ref());
+        assert_eq!(*buffer0.get_ref(), hex!(0000803F 00000040 00004040),);
     }
 
     #[test]
@@ -1522,7 +1505,7 @@ mod tests {
         )];
         write_attributes(&buffer_info, &mut [&mut buffer0], &[0]).unwrap();
 
-        assert_eq!(&hex_bytes("0000803F 00000040 00004040"), buffer0.get_ref());
+        assert_eq!(*buffer0.get_ref(), hex!(0000803F 00000040 00004040));
     }
 
     #[test]
@@ -1550,18 +1533,26 @@ mod tests {
         write_attributes(&buffer_info, &mut [&mut buffer0, &mut buffer1], &[4, 8]).unwrap();
 
         assert_eq!(
-            &hex_bytes(
-                "00000000 0000803F 0000803F 0000803F 00000040 00000040 00004040 00004040 00004040 00004040
-                          00000000 00000000 00000000 00000040 00000040 00004040 00004040 00004040 00004040"
-            ),
-            buffer0.get_ref()
+            *buffer0.get_ref(),
+            hex!(
+                // Offset
+                00000000
+                // Vertex 0
+                0000803F 0000803F 0000803F 00000040 00000040 00004040 00004040 00004040 00004040
+                // Vertex 1
+                00000000 00000000 00000000 00000040 00000040 00004040 00004040 00004040 00004040
+            )
         );
         assert_eq!(
-            &hex_bytes(
-                "00000000 00000000 0000803F 0000803F 00000040 00000040
-                                   00000000 00000000 00000040 00000040"
-            ),
-            buffer1.get_ref()
+            *buffer1.get_ref(),
+            hex!(
+                // Offset
+                00000000 00000000
+                // Vertex 0
+                0000803F 0000803F 00000040 00000040
+                // Vertex 1
+                00000000 00000000 00000040 00000040
+            )
         );
     }
 
@@ -1596,18 +1587,26 @@ mod tests {
         write_attributes(&buffer_info, &mut [&mut buffer0, &mut buffer1], &[4, 8]).unwrap();
 
         assert_eq!(
-            &hex_bytes(
-                "00000000 0000803F 0000803F 0000803F 00400040 00004040 00004040 00004040 00004040
-                          00000000 00000000 00000000 00400040 00004040 00004040 00004040 00004040"
+            *buffer0.get_ref(),
+            hex!(
+                // Offset
+                00000000
+                // Vertex 0
+                0000803F 0000803F 0000803F 00400040 00004040 00004040 00004040 00004040
+                // Vertex 1
+                00000000 00000000 00000000 00400040 00004040 00004040 00004040 00004040
             ),
-            buffer0.get_ref()
         );
         assert_eq!(
-            &hex_bytes(
-                "00000000 00000000 0000803F 0000803F 00000040 00000040
-                                   00000000 00000000 00000040 00000040"
+            *buffer1.get_ref(),
+            hex!(
+                // Offset
+                00000000 00000000
+                // Vertex 0
+                0000803F 0000803F 00000040 00000040
+                // Vertex 1
+                00000000 00000000 00000040 00000040
             ),
-            buffer1.get_ref()
         );
     }
 }
