@@ -1,4 +1,5 @@
 use std::{
+    collections::HashSet,
     convert::TryInto,
     io::{Read, Seek},
     path::Path,
@@ -9,17 +10,25 @@ use ssbh_lib::{
     formats::skel::{BillboardType, Skel, SkelBoneEntry, SkelEntryFlags},
     Matrix4x4,
 };
+use thiserror::Error;
 
-use crate::create_ssbh_array;
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+
+use crate::{create_ssbh_array, SsbhData};
 
 /// The data associated with a [Skel] file.
 /// The supported version is 1.0.
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[derive(Debug)]
 pub struct SkelData {
     pub major_version: u16,
     pub minor_version: u16,
     pub bones: Vec<BoneData>,
 }
 
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[derive(Debug)]
 pub struct BoneData {
     /// The name of the bone.
     pub name: String,
@@ -31,56 +40,80 @@ pub struct BoneData {
     // TODO: Flags?
 }
 
-impl SkelData {
-    /// Tries to read and convert the SKEL from `path`.
-    /// The entire file is buffered for performance.
-    pub fn from_file<P: AsRef<Path>>(path: P) -> Result<Self, Box<dyn std::error::Error>> {
+impl SsbhData for SkelData {
+    type WriteError = SkelError;
+
+    fn from_file<P: AsRef<Path>>(path: P) -> Result<Self, Box<dyn std::error::Error>> {
         let skel = Skel::from_file(path)?;
         Ok((&skel).into())
     }
 
-    /// Tries to read and convert the SKEL from `reader`.
-    /// For best performance when opening from a file, use `from_file` instead.
-    pub fn read<R: Read + Seek>(reader: &mut R) -> Result<Self, Box<dyn std::error::Error>> {
+    fn read<R: Read + Seek>(reader: &mut R) -> Result<Self, Box<dyn std::error::Error>> {
         let skel = Skel::read(reader)?;
         Ok((&skel).into())
     }
 
-    /// Converts the data to SKEL and writes to the given `writer`.
-    /// For best performance when writing to a file, use `write_to_file` instead.
-    pub fn write<W: std::io::Write + Seek>(&self, writer: &mut W) -> std::io::Result<()> {
-        let skel = create_skel(&self);
+    fn write<W: std::io::Write + Seek>(&self, writer: &mut W) -> Result<(), SkelError> {
+        let skel = create_skel(self)?;
         skel.write(writer)?;
         Ok(())
     }
 
-    /// Converts the data to SKEL and writes to the given `path`.
-    /// The entire file is buffered for performance.
-    pub fn write_to_file<P: AsRef<Path>>(&self, path: P) -> std::io::Result<()> {
-        let skel = create_skel(&self);
+    fn write_to_file<P: AsRef<Path>>(&self, path: P) -> Result<(), SkelError> {
+        let skel = create_skel(self)?;
         skel.write_to_file(path)?;
         Ok(())
     }
 }
 
+/// Errors while creating an [Skel] from [SkelData].
+#[derive(Error, Debug)]
+
+pub enum SkelError {
+    /// Creating a [Skel] file for the given version is not supported.
+    #[error(
+        "Creating a version {}.{} skel is not supported.",
+        major_version,
+        minor_version
+    )]
+    UnsupportedVersion {
+        major_version: u16,
+        minor_version: u16,
+    },
+
+    /// An error occurred while calculating a transformation matrix.
+    #[error(transparent)]
+    BoneTransform(#[from] BoneTransformError),
+
+    /// An error occurred while writing data to a buffer.
+    #[error(transparent)]
+    Io(#[from] std::io::Error),
+}
+
 /// Calculates the transform of `world_transform` relative to `parent_world_transform`.
-/// If `parent_world_transform` is [None] or the identity matrix, a copy of `world_transform` is returned.
+/// If `parent_world_transform` is [None], a copy of `world_transform` is returned.
 /// All matrices are assumed to be in row-major order.
+/// # Examples
+/// The simplest case is when the parent transform is the identity matrix,
+/// so the result is simply the passed in `world_transform`.
 /**
 ```rust
 # use ssbh_data::skel_data::calculate_relative_transform;
+// A row-major transform with a scale (2, 4, 8) and translation (1, 2, 3).
 let world_transform = [
     [2.0, 0.0, 0.0, 0.0],
     [0.0, 4.0, 0.0, 0.0],
     [0.0, 0.0, 8.0, 0.0],
     [1.0, 2.0, 3.0, 1.0],
 ];
+// The 4x4 identity matrix.
 let parent_world_transform = [
     [1.0, 0.0, 0.0, 0.0],
     [0.0, 1.0, 0.0, 0.0],
     [0.0, 0.0, 1.0, 0.0],
     [0.0, 0.0, 0.0, 1.0],
 ];
+
 assert_eq!(
     world_transform,
     calculate_relative_transform(
@@ -113,16 +146,15 @@ fn inv_transform(m: &[[f32; 4]; 4]) -> Matrix4x4 {
     Matrix4x4::from_rows_array(&inv)
 }
 
-// TODO: Can this fail?
-pub fn create_skel(data: &SkelData) -> Skel {
-    let world_transforms: Vec<_> = data
+fn create_skel(data: &SkelData) -> Result<Skel, SkelError> {
+    let world_transforms = data
         .bones
         .iter()
         .map(|b| data.calculate_world_transform(b))
-        .collect();
+        .collect::<Result<Vec<_>, _>>()?;
 
     // TODO: Add a test for this with a few bones.
-    Skel {
+    Ok(Skel {
         major_version: data.major_version,
         minor_version: data.minor_version,
         bone_entries: data
@@ -148,15 +180,15 @@ pub fn create_skel(data: &SkelData) -> Skel {
         inv_world_transforms: create_ssbh_array(&world_transforms, inv_transform),
         transforms: create_ssbh_array(&data.bones, |b| Matrix4x4::from_rows_array(&b.transform)),
         inv_transforms: create_ssbh_array(&data.bones, |b| inv_transform(&b.transform)),
-    }
+    })
 }
 
 impl From<&Skel> for SkelData {
-    // TODO: Add additional validation for mismatched array lengths?
     fn from(skel: &Skel) -> Self {
         Self {
             major_version: skel.major_version,
             minor_version: skel.minor_version,
+            // TODO: Add additional validation for mismatched array lengths?
             bones: skel
                 .bone_entries
                 .elements
@@ -177,13 +209,14 @@ fn create_bone_data(b: &SkelBoneEntry, transform: &Matrix4x4) -> BoneData {
 }
 
 fn mat4_from_row2d(elements: &[[f32; 4]; 4]) -> Mat4 {
-    Mat4::from_cols_array_2d(&elements).transpose()
+    Mat4::from_cols_array_2d(elements).transpose()
 }
 
 impl SkelData {
     /// Calculates the world transform for `bone` by accumulating the transform with the parents transform recursively.
-    /// For single bound objects, the object is transformed by the parent bone's world transform.
     /// Returns the resulting matrix in row-major order.
+    /// # Examples
+    /// For single bound mesh objects, the object is transformed by the parent bone's world transform.
     /**
     ```rust
     # use ssbh_data::skel_data::{BoneData, SkelData};
@@ -199,15 +232,27 @@ impl SkelData {
     let parent_bone_name = "Head";
     if let Some(parent_bone) = data.bones.iter().find(|b| b.name == parent_bone_name) {
         let world_transform = data.calculate_world_transform(&parent_bone);
+        // Transform the object using the matrix...
     }
     ```
     */
-    pub fn calculate_world_transform(&self, bone: &BoneData) -> [[f32; 4]; 4] {
+    pub fn calculate_world_transform(
+        &self,
+        bone: &BoneData,
+    ) -> Result<[[f32; 4]; 4], BoneTransformError> {
         let mut bone = bone;
         let mut transform = mat4_from_row2d(&bone.transform);
 
+        // Check for cycles by keeping track of previously visited locations.
+        let mut visited = HashSet::new();
+
         // Accumulate transforms by travelling up the bone heirarchy.
         while let Some(parent_index) = bone.parent_index {
+            if !visited.insert(parent_index) {
+                return Err(BoneTransformError::CycleDetected {
+                    index: parent_index,
+                });
+            }
             if let Some(parent_bone) = self.bones.get(parent_index) {
                 let parent_transform = mat4_from_row2d(&parent_bone.transform);
                 transform = transform.mul_mat4(&parent_transform);
@@ -218,10 +263,20 @@ impl SkelData {
         }
 
         // Save the result in row-major order.
-        transform.transpose().to_cols_array_2d()
+        Ok(transform.transpose().to_cols_array_2d())
     }
 }
 
+/// Errors while calculating [BoneData] transformation matrices.
+#[derive(Error, Debug)]
+pub enum BoneTransformError {
+    #[error(
+        "Cyclical bone chains are not supported. A cycle was detected at index {}.",
+        index
+    )]
+    CycleDetected { index: usize },
+}
+
 #[cfg(test)]
 mod tests {
     use approx::relative_eq;
@@ -350,7 +405,67 @@ mod tests {
             }],
         };
 
-        assert_eq!(transform, data.calculate_world_transform(&data.bones[0]));
+        assert_eq!(
+            transform,
+            data.calculate_world_transform(&data.bones[0]).unwrap()
+        );
+    }
+
+    #[test]
+    fn world_transform_self_referential_bone() {
+        let data = SkelData {
+            major_version: 1,
+            minor_version: 0,
+            bones: vec![BoneData {
+                name: "root".to_string(),
+                transform: [[0.0; 4]; 4],
+                parent_index: Some(0),
+            }],
+        };
+
+        // This should still terminate.
+        let result = data.calculate_world_transform(&data.bones[0]);
+        assert!(matches!(
+            result,
+            Err(BoneTransformError::CycleDetected { index: 0 })
+        ));
+    }
+
+    #[test]
+    fn world_transform_bone_cycle() {
+        let data = SkelData {
+            major_version: 1,
+            minor_version: 0,
+            bones: vec![
+                BoneData {
+                    name: "a".to_string(),
+                    transform: [[0.0; 4]; 4],
+                    parent_index: None,
+                },
+                BoneData {
+                    name: "b".to_string(),
+                    transform: [[0.0; 4]; 4],
+                    parent_index: Some(2),
+                },
+                BoneData {
+                    name: "c".to_string(),
+                    transform: [[0.0; 4]; 4],
+                    parent_index: Some(1),
+                },
+                BoneData {
+                    name: "d".to_string(),
+                    transform: [[0.0; 4]; 4],
+                    parent_index: Some(2),
+                },
+            ],
+        };
+
+        // This should still terminate.
+        let result = data.calculate_world_transform(&data.bones[2]);
+        assert!(matches!(
+            result,
+            Err(BoneTransformError::CycleDetected { index: 1 })
+        ));
     }
 
     #[test]
@@ -410,7 +525,7 @@ mod tests {
                 [1.0, 0.0, 0.0, 0.0],
                 [0.0, 12.6236, 0.268775, 1.0]
             ],
-            data.calculate_world_transform(&data.bones[3]),
+            data.calculate_world_transform(&data.bones[3]).unwrap(),
         ));
     }
 }
